@@ -9,6 +9,11 @@
 #' @param free_translation character vector of the length one for the free translation line.
 #' @param annotation character vector of the length one for the annotation line (above translation).
 #' @param grammaticality character vector with the grammaticality value.
+#' @param audio_path character string with the path to the sound in .wav format.
+#' @param audio_label character string for the label to display.
+#' @param video_path character string with the path to the video.
+#' @param video_width width argument for the video in px.
+#' @param video_height height argument for the video in px.
 #' @param comment character vector of the length one for the comment line (under the free translation line).
 #' @param line_length integer vector of the length one that denotes maximum number of characters per one line.
 #' @param italic_transliteration logical variable that denotes, whether user wants to italicize your example.
@@ -30,6 +35,7 @@
 #'               intext = TRUE)
 #'
 #' @importFrom knitr is_latex_output
+#' @importFrom knitr is_html_output
 #' @importFrom kableExtra kable_minimal
 #' @importFrom kableExtra kbl
 #' @importFrom kableExtra footnote
@@ -41,6 +47,11 @@ gloss_example <- function(transliteration,
                           comment = "",
                           annotation = NULL,
                           grammaticality = NULL,
+                          audio_path = NULL,
+                          audio_label = "\u266A",
+                          video_path = NULL,
+                          video_width = 320,
+                          video_height = 240,
                           line_length = 70,
                           italic_transliteration = TRUE,
                           drop_transliteration = FALSE,
@@ -63,10 +74,23 @@ gloss_example <- function(transliteration,
                 " argument should be a character vector of length 1"))
   }
 
-# fix the apostrophe problem
+# fix the apostrophe and "> <" problem
   if(!drop_transliteration){
-    transliteration <- gsub(pattern = "[\u2019\u02BC]", replacement = "'", transliteration)}
+    transliteration <- gsub(pattern = "[\u2019\u02BC]", replacement = "'", transliteration)
+    transliteration <- gsub(pattern = "<", replacement = "&lt;", transliteration)
+    transliteration <- gsub(pattern = ">", replacement = "&gt;", transliteration)
+    transliteration <- gsub(pattern = "!\\[\\]\\(", replacement = "pictures_inside_turn_me_back_please", transliteration)
+    transliteration <- gsub(pattern = "\\[", replacement = "\uFF3B", transliteration)
+    transliteration <- gsub(pattern = "\\]", replacement = "\uFF3D", transliteration)
+    transliteration <- gsub(pattern = "pictures_inside_turn_me_back_please", replacement = "!\\[\\]\\(", transliteration)
+    }
   glosses <- gsub(pattern = "[\u2019\u02BC]", replacement = "'", glosses)
+  glosses <- gsub(pattern = "!\\[\\]\\(", replacement = "pictures_inside_turn_me_back_please", glosses)
+  glosses <- gsub(pattern = "\\[", replacement = "\uFF3B", glosses)
+  glosses <- gsub(pattern = "\\]", replacement = "\uFF3D", glosses)
+  glosses <- gsub(pattern = "pictures_inside_turn_me_back_please", replacement = "!\\[\\]\\(", glosses)
+
+
   if(!is.null(grammaticality)){
     grammaticality <- gsub(pattern = "\\*", replacement = "\uFF0A", grammaticality)
   }
@@ -79,6 +103,10 @@ gloss_example <- function(transliteration,
   transliteration <- unlist(strsplit(transliteration, " "))
   glosses_by_word <- unlist(strsplit(glosses, " "))
 
+  if(!is.null(annotation)){
+    annotation <- unlist(strsplit(annotation, " "))
+  }
+
 # prepare vector of splits of the glosses by line --------------------------
   longest <- if(sum(nchar(transliteration)) > sum(nchar(glosses))){
     transliteration
@@ -99,15 +127,18 @@ gloss_example <- function(transliteration,
   }
 
 # add glosses to the document gloss list -----------------------------------
-  single_gl <- unlist(strsplit(glosses_by_word, "[-\\.=:\\)\\(!\\?]"))
+  single_gl <- unlist(strsplit(glosses_by_word, "[-\\.=:\\)\\(!\\?<>\\~\uFF3D\uFF3B]"))
   starts_with_punctuation <- single_gl[1] == ""
+  single_gl <- gsub(pattern = "<", replacement = "&lt;", single_gl)
+  single_gl <- gsub(pattern = ">", replacement = "&gt;", single_gl)
   single_gl <- lingglosses::add_gloss(single_gl)
   if(starts_with_punctuation){single_gl <- c("", single_gl)}
 
 # get delimiters back ------------------------------------------------------
   delimiters <- unlist(strsplit(glosses,
-"[^-:\\.= \\)\\(!\\?\u201E\u201C\u2019\u201D\u00BB\u00AB\u201F]"))
+"[^-:\\.= \\)\\(!\\?\u201E\u201C\u2019\u201D\u00BB\u00AB\u201F<>\\~\uFF3B\uFF3D]"))
   delimiters <- c(delimiters[delimiters != ""], "")
+  if(!starts_with_punctuation){single_gl <- c(single_gl, rep("", sum(delimiters == ">")))}
   glosses <- paste0(single_gl, delimiters, collapse = "")
   glosses <- gsub("<span style=", "<span_style=", glosses)
   glosses <- unlist(strsplit(glosses, " "))
@@ -118,7 +149,7 @@ gloss_example <- function(transliteration,
     if(knitr::is_latex_output()){
       transliteration <- paste0("\\textit{", transliteration, "}")
     } else {
-      transliteration <- paste0("*", transliteration, "*")
+      transliteration <- paste0("_", transliteration, "_")
     }
   }
 
@@ -148,23 +179,27 @@ gloss_example <- function(transliteration,
                        paste0(" ", comment)} else {""},
                      sep2,
                      collapse = "")
-  } else{
+  } else {
 
 # long line splitting ------------------------------------------------------
 
     if(length(unique(splits_by_line)) > 1){
       multiline_result <- lapply(unique(splits_by_line), function(i){
         gloss_example(
-          paste(transliteration[splits_by_line == i], collapse = " "),
-          paste(glosses_by_word[splits_by_line == i], collapse = " "),
+          transliteration = paste(transliteration[splits_by_line == i],
+                                  collapse = " "),
+          glosses = paste(glosses_by_word[splits_by_line == i], collapse = " "),
           free_translation = if(i == max(splits_by_line)){free_translation} else {""},
           grammaticality = if(i == min(splits_by_line)){grammaticality} else {NULL},
           annotation = if(!is.null(annotation)){
-            paste(annotation[splits_by_line == i], collapse = " ")} else {""},
+            paste(annotation[splits_by_line == i], collapse = " ")} else {NULL},
           comment = if(i == max(splits_by_line)){comment} else {""},
           italic_transliteration = FALSE,
-          line_length = line_length,
+          line_length = nchar(paste(glosses_by_word[splits_by_line == i], collapse = " "))+1,
           drop_transliteration = drop_transliteration,
+          audio_path = if(i == max(splits_by_line)){audio_path} else {NULL},
+          audio_label = audio_label,
+          video_path = if(i == max(splits_by_line)){video_path} else {NULL},
           intext = FALSE,
           write_to_db = FALSE)
       })
@@ -189,13 +224,28 @@ gloss_example <- function(transliteration,
           result <- cbind(matrix(c("", grammaticality, "")), result)
         }
       }
-
       result <- kableExtra::kbl(result, align = "l", centering = FALSE,
                                 escape = FALSE, vline = "")
       result <- kableExtra::kable_minimal(result,
                                           position = "left",
                                           full_width = FALSE)
 
+
+# add video ---------------------------------------------------------------
+      if(!is.null(video_path) & knitr::is_html_output()){
+        if(length(video_path) > 1){
+          stop("video_path argument should be of the length 1")
+        }
+        result <- kableExtra::footnote(kable_input = result,
+                                       general = as.character(
+                                         add_video(video_path,
+                                                   video_width,
+                                                   video_height)),
+                                       general_title = "",
+                                       escape = FALSE)
+      }
+
+
 # add comment --------------------------------------------------------------
       if(nchar(comment) > 0){
         result <- kableExtra::footnote(kable_input = result,
@@ -203,11 +253,29 @@ gloss_example <- function(transliteration,
                                        general_title = "")
       }
 
+# add audio ---------------------------------------------------------------
+      if(!is.null(audio_path) & knitr::is_html_output()){
+        if(length(audio_path) > 1){
+          stop("audio_path argument should be of the length 1")
+        }
+        if(length(audio_label) > 1){
+          stop("audio_label argument should be of the length 1")
+        }
+        # if(!file.exists(audio_path)){
+        #   stop(paste("It look like there is no file", audio_path))
+        # }
+        add_to_translation <- paste("'", add_sound(audio_path, audio_label))
+      } else {
+        add_to_translation <- "'"
+      }
 # add free translation -----------------------------------------------------
       if(nchar(free_translation) > 0){
         result <- kableExtra::footnote(kable_input = result,
-                                       general = paste0("'", free_translation, "'"),
-                                       general_title = "")
+                                       general = paste0("'",
+                                                        free_translation,
+                                                        add_to_translation),
+                                       general_title = "",
+                                       escape = FALSE)
       }
 
 # remove lines from LaTeX --------------------------------------------------
